# -*- coding: utf-8 -*-

"""
MEDIA:
    Diario de Chiapas, Chiapas

USAGE
    $ cd diarioDeChiapas
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date. ##
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific date. ##
    $ scrapy crawl noticias --nolog -s filename=2018-08-30.json -a year=2018 -a month=8 -a day=30
"""

import scrapy, re, json
from datetime import datetime, date
from diarioDeChiapas.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class ImportantData(scrapy.Item):
    """
    Useful data for the flow of the implementation
    """
    to_next_page = scrapy.Field()
    is_last_link = scrapy.Field()
    news_section = scrapy.Field()
    return_url   = scrapy.Field()



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"

    def start_requests(self):
        year  = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day   = getattr(self, "day", None)

        if year is not None and month is not None and day is not None:
            self.stopDate = date(int(year), int(month), int(day))
        else:
            self.stopDate = None

        baseURL = "http://www.diariodechiapas.com/landing/"

        section_list = ["editorial", "portada", "metropoli", "region", "la-roja",
                       "deportes", "boga", "ae", "trascendio"]
        # section_list = ["editorial"]

        if self.stopDate is None:
            for s in section_list:
                yield scrapy.Request(url=baseURL + s, callback=self.parse)

        else:
            for s in section_list:
                flow_info = ImportantData()
                flow_info['to_next_page'] = False
                
                request = scrapy.Request(url=baseURL + s, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request



    def parse(self, response):
        link_list = response.xpath('//section[@class="page__content"]').css('section.post').xpath('./a[@class="post__link"]/@href').extract()
        section   = response.xpath('//section[@class="wrapper"]/h1').extract_first()
        if section is not None : section = remove_tags(section)

        for link in link_list:
            flow_info = ImportantData()
            flow_info['news_section'] = section

            request = scrapy.Request(url=link, callback=self.parse_item)
            request.meta['item'] = flow_info

            yield request

        next_page = response.css('div.wp-pagenavi').css('a.nextpostslink').css('::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_with_stop_date(self, response):
        flow_info = response.meta['item']

        if not flow_info['to_next_page']:
            link_list = response.xpath('//section[@class="page__content"]').css('section.post').xpath('./a[@class="post__link"]/@href').extract()
            section   = response.xpath('//section[@class="wrapper"]/h1').extract_first()
            if section is not None : section = remove_tags(section)

            for link in link_list:
                flow_info = ImportantData()
                flow_info['news_section'] = section
                flow_info['return_url'] = response.url
                
                if link == link_list[-1] : flow_info['is_last_link'] = True
                else                     : flow_info['is_last_link'] = False
                
                request = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request

        else:
            next_page = response.css('div.wp-pagenavi').css('a.nextpostslink').css('::attr(href)').extract_first()
            if next_page is not None:
                flow_info['to_next_page'] = False
                
                request = scrapy.Request(url=next_page, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request



    def parse_item(self, response):
        flow_info = response.meta['item']
        item = NoticiasItem()
        text = ''

        title = response.xpath('//section[@class="single__content"]/h1').extract_first()
        if title is not None : title = remove_tags(title)

        for p in response.xpath('//section[@class="single__content"]').css('p').extract():
            text += remove_tags(p) + "\n"

        ## News item info ##
        item['date']  = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        item['topic'] = flow_info['news_section']
        item['title'] = title
        item['text']  = text.strip()
        item['url']   = response.url

        yield item



    def parse_item_with_stop_date(self, response):
        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        news_date = news_date[:news_date.find('T')]
        news_date = datetime.strptime(news_date, '%Y-%m-%d').date()

        if news_date >= self.stopDate:
            flow_info = response.meta['item']
            item = NoticiasItem()
            text = ''

            title = response.xpath('//section[@class="single__content"]/h1').extract_first()
            if title is not None : title = remove_tags(title)

            for p in response.xpath('//section[@class="single__content"]').css('p').extract():
                text += remove_tags(p) + "\n"

            ## News item info ##
            item['date']  = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
            item['topic'] = flow_info['news_section']
            item['title'] = title
            item['text']  = text.strip()
            item['url']   = response.url

            yield item

            if flow_info['is_last_link']:
                flow_info['to_next_page'] = True
                
                request = scrapy.Request(url=flow_info['return_url'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = flow_info
                
                yield request
