# -*- coding: utf-8 -*-

"""
MEDIA:
    El Heraldo de Chihuahua, Chihuahua

USAGE
    $ cd heraldoChihuahua
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date. ##
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific date. ##
    $ scrapy crawl noticias --nolog -s filename=2018-08-30.json -a year=2018 -a month=8 -a day=30
"""

import scrapy, re, json, urllib
from datetime import datetime, date, tzinfo, timedelta
from collections import OrderedDict
from heraldoChihuahua.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


DAY_NUM = re.compile(r'[0-9]{1,2}')


class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for Chihuahua: UTC-7 ##
        return timedelta(hours=-7)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-7'



class ImportantData(scrapy.Item):
    """
    Useful data for the flow of the implementation
    """
    to_next_page = scrapy.Field()
    is_last_link = scrapy.Field()
    news_section = scrapy.Field()
    news_page    = scrapy.Field()
    return_url   = scrapy.Field()



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"

    def start_requests(self):
        self.tz = UTC()
        year    = getattr(self, "year", None)
        month   = getattr(self, "month", None)
        day     = getattr(self, "day", None)

        if year is not None and month is not None and day is not None:
            self.stopDate = date(int(year), int(month), int(day))
        else:
            self.stopDate = None

        self.baseURL = "https://www.elheraldodechihuahua.com.mx/"

        section_list = ["local", "policiaca", "mexico", "republica", "mundo", "finanzas",
                        "analisis", "gossip", "circulos", "cultura", "doble-via", "deportes"]
        # section_list = ["local"]

        self.month_parser = dict(enero='01', febrero='02', marzo='03',      abril='04',   mayo='05',      junio='06',
                                 julio='07', agosto='08',  septiembre='09', octubre='10', noviembre='11', diciembre='12')

        if self.stopDate is None:
            for s in section_list:
                flow_info = ImportantData()
                flow_info['news_page'] = 8
                flow_info['news_section'] = s

                request = scrapy.Request(url=self.baseURL + s, callback=self.parse)
                request.meta['item'] = flow_info

                yield request

        else:
            for s in section_list:
                flow_info = ImportantData()
                flow_info['to_next_page'] = False
                flow_info['news_page'] = 8
                flow_info['news_section'] = s
                
                request = scrapy.Request(url=self.baseURL + s, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request



    def parse(self, response):
        flow_info = response.meta['item']

        link_list = response.css('div.hard-news-row').css('h4.title > a::attr(href)').extract()
        if len(link_list) <= 0 : link_list = response.css('h4.title > a::attr(href)').extract()

        for link in link_list:
            yield scrapy.Request(url=link, callback=self.parse_item)

        load_more = response.css('div.load-more').extract_first()
        if load_more is not None:
            flow_info['news_page'] += 4

            params = OrderedDict()
            params['widgetContentId'] = '148'
            params['widgetName']      = 'viewPicker'
            params['offset']          = str(flow_info['news_page'])

            url_params = urllib.urlencode(params)
            next_load = self.baseURL + flow_info['news_section'] + "/widget/?" + url_params
            
            request = scrapy.Request(url=next_load, callback=self.parse)
            request.meta['item'] = flow_info

            yield request



    def parse_with_stop_date(self, response):
        flow_info = response.meta['item']

        if not flow_info['to_next_page']:
            link_list = response.css('div.hard-news-row').css('h4.title > a::attr(href)').extract()
            if len(link_list) <= 0 : link_list = response.css('h4.title > a::attr(href)').extract()
            news_page    = flow_info['news_page']
            news_section = flow_info['news_section']

            for link in link_list:
                flow_info = ImportantData()
                flow_info['return_url']   = response.url
                flow_info['news_page']    = news_page
                flow_info['news_section'] = news_section
                
                if link == link_list[-1] : flow_info['is_last_link'] = True
                else                     : flow_info['is_last_link'] = False
                
                request = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request

        else:
            load_more = response.css('div.load-more').extract_first()
            if load_more is not None:
                flow_info['to_next_page'] = False
                flow_info['news_page'] += 4

                params = OrderedDict()
                params['widgetContentId'] = '148'
                params['widgetName']      = 'viewPicker'
                params['offset']          = str(flow_info['news_page'])

                url_params = urllib.urlencode(params)
                next_load = self.baseURL + flow_info['news_section'] + "/widget/?" + url_params
                
                request = scrapy.Request(url=next_load, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request



    def parse_item(self, response):
        published_date = response.css('p.published-date').extract_first()  
        if published_date is not None:
            published_date = remove_tags(published_date)
            published_date = published_date.split('/')
            published_date = published_date[1].strip().replace('\n', '')
            date_lst = published_date.split(' de ')
            element = DAY_NUM.search(date_lst[0])
            
            if element:
                item = NoticiasItem()
                text = ''

                date_lst[0] = element.group()
                date_lst[1] = self.month_parser[date_lst[1].lower()]
                date_lst = map(int, date_lst)
                news_date = datetime(date_lst[2], date_lst[1], date_lst[0], tzinfo=self.tz).isoformat("T")

                topic = response.css('div.breadcrumb > a').extract_first()
                if topic is not None : topic = remove_tags(topic).strip("\n")

                title = response.css('h1.title').extract_first()
                if title is not None : title = remove_tags(title).strip("\n")

                for p in response.css('div.content-body').xpath('./div[contains(@id, "content-body")]/p').extract():
                    text += remove_tags(p) + "\n"

                ## News item info ##
                item['date']  = news_date
                item['topic'] = topic
                item['title'] = title
                item['text']  = text.strip()
                item['url']   = response.url

                yield item



    def parse_item_with_stop_date(self, response):
        published_date = response.css('p.published-date').extract_first()
        
        if published_date is not None:
            published_date = remove_tags(published_date)
            published_date = published_date.split('/')
            published_date = published_date[1].strip().replace('\n', '')
            date_lst = published_date.split(' de ')
            element = DAY_NUM.search(date_lst[0])
            
            if element:
                date_lst[0] = element.group()
                date_lst[1] = self.month_parser[date_lst[1].lower()]
                news_date = "-".join(date_lst)
                news_date = datetime.strptime(news_date, '%d-%m-%Y').date()

                if news_date >= self.stopDate:
                    flow_info = response.meta['item']
                    item = NoticiasItem()
                    text = ''

                    date_lst = map(int, date_lst)
                    news_date = datetime(date_lst[2], date_lst[1], date_lst[0], tzinfo=self.tz).isoformat("T")

                    topic = response.css('div.breadcrumb > a').extract_first()
                    if topic is not None : topic = remove_tags(topic).strip("\n")

                    title = response.css('h1.title').extract_first()
                    if title is not None : title = remove_tags(title).strip("\n")

                    for p in response.css('div.content-body').xpath('./div[contains(@id, "content-body")]/p').extract():
                        text += remove_tags(p) + "\n"

                    ## News item info ##
                    item['date']  = news_date
                    item['topic'] = topic
                    item['title'] = title
                    item['text']  = text.strip()
                    item['url']   = response.url

                    yield item

                    if flow_info['is_last_link']:
                        flow_info['to_next_page'] = True
                        
                        request = scrapy.Request(url=flow_info['return_url'], callback=self.parse_with_stop_date, dont_filter=True)
                        request.meta['item'] = flow_info
                        
                        yield request
