# -*- coding: utf-8 -*-

"""
MEDIA:
    Tinta Fresca, Chiapas

USAGE
    $ cd tintaFresca
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date. ##
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific date. ##
    $ scrapy crawl noticias --nolog -s filename=2018-08-30.json -a year=2018 -a month=8 -a day=30
"""

import scrapy, re, json
from datetime import datetime, date, tzinfo, timedelta
from tintaFresca.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for Chiapas: UTC-6 ##
        return timedelta(hours=-6)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-6'



class ImportantData(scrapy.Item):
    """
    Useful data for the flow of the implementation
    """
    to_next_page = scrapy.Field()
    is_last_link = scrapy.Field()
    news_section = scrapy.Field()
    return_url   = scrapy.Field()



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"

    def start_requests(self):
        self.tz = UTC()
        year    = getattr(self, "year", None)
        month   = getattr(self, "month", None)
        day     = getattr(self, "day", None)

        if year is not None and month is not None and day is not None:
            self.stopDate = date(int(year), int(month), int(day))
        else:
            self.stopDate = None

        baseURL = "http://tintafresca.com.mx/"

        # section_list = ["letras_en_su_tinta/page1/", "tgz/page1/", "patria_chica/page1/", "hecho_en_chiapas/page1/", "show/page1/", "rafaga/page1/"]
        section_list = ["tgz/page1/", "patria_chica/page1/", "hecho_en_chiapas/page1/", "show/page1/"]

        self.month_parser = dict(Enero='01', Febrero='02', Marzo='03',      Abril='04',   Mayo='05',      Junio='06',
                                 Julio='07', Agosto='08',  Septiembre='09', Octubre='10', Noviembre='11', Diciembre='12')

        if self.stopDate is None:
            for s in section_list:
                yield scrapy.Request(url=baseURL + s, callback=self.parse)

        else:
            for s in section_list:
                flow_info = ImportantData()
                flow_info['to_next_page'] = False
                
                request = scrapy.Request(url=baseURL + s, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request



    def parse(self, response):
        link_list = response.css('ul.tintas').css('a.leer::attr(href)').extract()
        # section   = response.xpath('//div[@id="ruta"]').extract_first()
        
        # if section is not None:
        #     section = remove_tags(section)
        #     section = section.replace("Inicio &gt; ", '')

        for link in link_list:
            flow_info = ImportantData()
            # flow_info['news_section'] = section

            request = scrapy.Request(url=link, callback=self.parse_item)
            request.meta['item'] = flow_info

            yield request

        next_page = response.xpath('//ul[@class="pagination"]/li[3]/a/@href').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_with_stop_date(self, response):
        flow_info = response.meta['item']

        if not flow_info['to_next_page']:
            link_list = response.css('ul.tintas').css('a.leer::attr(href)').extract()
            # section   = response.xpath('//div[@id="ruta"]').extract_first()
            
            # if section is not None:
            #     section = remove_tags(section)
            #     section = section.replace("Inicio &gt; ", '')

            for link in link_list:
                flow_info = ImportantData()
                # flow_info['news_section'] = section
                flow_info['return_url'] = response.url
                
                if link == link_list[-1] : flow_info['is_last_link'] = True
                else                     : flow_info['is_last_link'] = False
                
                request = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request

        else:
            next_page = response.xpath('//ul[@class="pagination"]/li[3]/a/@href').extract_first()
            if next_page is not None:
                flow_info['to_next_page'] = False
                
                request = scrapy.Request(url=next_page, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request



    def parse_item(self, response):
        flow_info = response.meta['item']
        item = NoticiasItem()
        text = ''

        date_str = response.xpath('//div[@class="balazo"]').extract_first()
        if date_str.find('<br>') > -1 : date_str = date_str[date_str.find('<br>'):]
        date_str = remove_tags(date_str)
        date_lst = date_str.split('/')
        date_lst[1] = self.month_parser[date_lst[1]]
        date_lst = map(int, date_lst)
        news_date = datetime(date_lst[2], date_lst[1], date_lst[0], tzinfo=self.tz).isoformat("T")

        topic = response.css('div.seccion > h3.left > a').extract_first()
        if topic is not None : topic = remove_tags(topic)

        if topic is not None:
            if topic == "Letras en su Tinta":
                title = ''
                lines = response.css('div.sumario > p').extract()

                for line in lines:
                    if lines.index(line) != len(lines)-1 : title += remove_tags(line) + ". "
                    else                                 : title += remove_tags(line) + "."
            else:
                title = response.css('div.titulo > h1').extract_first()
                if title is not None : title = remove_tags(title)
        else:
            title = None

        for p in response.css('div.contenido > p').extract():
            text += remove_tags(p) + "\n"

        ## News item info ##
        item['date']  = news_date
        item['topic'] = topic
        item['title'] = title
        item['text']  = text.strip()
        item['url']   = response.url

        yield item



    def parse_item_with_stop_date(self, response):
        date_str = response.xpath('//div[@class="balazo"]').extract_first()
        if date_str.find('<br>') > -1 : date_str = date_str[date_str.find('<br>'):]
        date_str = remove_tags(date_str)
        date_lst = date_str.split('/')
        date_lst[1] = self.month_parser[date_lst[1]]
        news_date = "-".join(date_lst)
        news_date = datetime.strptime(news_date, '%d-%m-%Y').date()

        if news_date >= self.stopDate:
            flow_info = response.meta['item']
            item = NoticiasItem()
            text = ''

            date_lst = map(int, date_lst)
            news_date = datetime(date_lst[2], date_lst[1], date_lst[0], tzinfo=self.tz).isoformat("T")

            topic = response.css('div.seccion > h3.left > a').extract_first()
            if topic is not None : topic = remove_tags(topic)

            if topic is not None:
                if topic == "Letras en su Tinta":
                    title = ''
                    lines = response.css('div.sumario > p').extract()

                    for line in lines:
                        if lines.index(line) != len(lines)-1 : title += remove_tags(line) + ". "
                        else                                 : title += remove_tags(line) + "."
                else:
                    title = response.css('div.titulo > h1').extract_first()
                    if title is not None : title = remove_tags(title)
            else:
                title = None

            for p in response.css('div.contenido > p').extract():
                text += remove_tags(p) + "\n"

            ## News item info ##
            item['date']  = news_date
            item['topic'] = topic
            item['title'] = title
            item['text']  = text.strip()
            item['url']   = response.url

            yield item

            if flow_info['is_last_link']:
                flow_info['to_next_page'] = True
                
                request = scrapy.Request(url=flow_info['return_url'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = flow_info
                
                yield request
